@@ -6,7 +6,9 @@
 {-# LANGUAGE FlexibleInstances          #-}
 {-# LANGUAGE GADTs                      #-}
 {-# LANGUAGE GeneralizedNewtypeDeriving #-}
+{-# LANGUAGE InstanceSigs               #-}
 {-# LANGUAGE MultiParamTypeClasses      #-}
+{-# LANGUAGE MultiWayIf                 #-}
 {-# LANGUAGE NondecreasingIndentation   #-}
 {-# LANGUAGE OverloadedStrings          #-}
 {-# LANGUAGE ScopedTypeVariables        #-}
@@ -56,44 +58,48 @@
 -- type of option records and rendering results specific to any
 -- particular backend.  For @b ~ SVG@, @v ~ V2@, we have
 --
--- > data Options SVG V2 n = SVGOptions
--- >     { _size           :: SizeSpec V2 n   -- ^ The requested size.
--- >     , _svgDefinitions :: Maybe SvgM
--- >                           -- ^ Custom definitions that will be added to the @defs@
--- >                           --   section of the output.
--- >     , _idPrefix       :: T.Text
--- >     }
+-- >data    Options SVG V2 n = SVGOptions
+-- >    { _size            :: SizeSpec V2 n   -- ^ The requested size.
+-- >    , _svgDefinitions  :: Maybe Element
+-- >                          -- ^ Custom definitions that will be added to the @defs@
+-- >                          --   section of the output.
+-- >    , _idPrefix        :: T.Text
+-- >    , _svgAttributes   :: [Attribute]
+-- >                          -- ^ Attriubtes to apply to the entire svg element.
+-- >    , _generateDoctype :: Bool
+-- >    }
 --
 -- @
 -- data family Render SVG V2 n = R 'SvgRenderM n'
 -- @
 --
 -- @
--- type family Result SVG V2 n = 'Graphics.Rendering.SVG.SvgM'
+-- type family Result SVG V2 n = 'Element'
 -- @
 --
 -- So the type of 'renderDia' resolves to
 --
 -- @
--- renderDia :: SVG -> Options SVG V2 n -> QDiagram SVG V2 n m -> 'Graphics.Rendering.SVG.SvgM'
+-- renderDia :: SVG -> Options SVG V2 n -> QDiagram SVG V2 n m -> 'Graphics.Rendering.SVG.Element'
 -- @
 --
 -- which you could call like @renderDia SVG (SVGOptions (mkWidth 250)
--- Nothing "") myDiagram@ (if you have the 'OverloadedStrings' extension
+-- Nothing "" [] True) myDiagram@ (if you have the 'OverloadedStrings' extension
 -- enabled; otherwise you can use 'Text.pack ""').  (In some
 -- situations GHC may not be able to infer the type @m@, in which case
 -- you can use a type annotation to specify it; it may be useful to
 -- simply use the type synonym @Diagram SVG = QDiagram SVG V2 Double
--- Any@.) This returns an 'Graphics.Rendering.SVG.SvgM' value, which
--- you can, /e.g./ render to a 'ByteString' using 'Lucid.Svg.renderBS'
--- from the 'lucid' package.
+-- Any@.) This returns an 'Graphics.Svg.Core.Element' value, which
+-- you can, /e.g./ render to a 'ByteString' using 'Graphics.Svg.Core.renderBS'
+-- from the 'svg-builder' package.
 --
 -----------------------------------------------------------------------------
 
 module Diagrams.Backend.SVG
   ( SVG(..) -- rendering token
   , B
-  , Options(..), sizeSpec, svgDefinitions, idPrefix -- for rendering options specific to SVG
+    -- for rendering options specific to SVG
+  , Options(..), sizeSpec, svgDefinitions, idPrefix, svgAttributes, generateDoctype
   , SVGFloat
 
   , renderSVG
@@ -142,11 +148,11 @@ import           Diagrams.TwoD.Attributes (splitTextureFills)
 import           Diagrams.TwoD.Path       (Clip (Clip))
 import           Diagrams.TwoD.Text
 
--- from lucid-svg
-import           Lucid.Svg
+-- from svg-builder
+import           Graphics.Svg             hiding ((<>))
 
 -- from this package
-import           Graphics.Rendering.SVG   (SVGFloat, SvgM)
+import           Graphics.Rendering.SVG   (SVGFloat)
 import qualified Graphics.Rendering.SVG   as R
 
 -- | @SVG@ is simply a token used to identify this rendering backend
@@ -182,13 +188,13 @@ initialSvgRenderState :: SvgRenderState
 initialSvgRenderState = SvgRenderState 0 0 1
 
 -- | Monad to keep track of environment and state when rendering an SVG.
-type SvgRenderM n = ReaderT (Environment n) (State SvgRenderState) SvgM
+type SvgRenderM n = ReaderT (Environment n) (State SvgRenderState) Element
 
-runRenderM :: SVGFloat n => T.Text -> SvgRenderM n -> SvgM
+runRenderM :: SVGFloat n => T.Text -> SvgRenderM n -> Element
 runRenderM o s = flip evalState initialSvgRenderState
                $ runReaderT  s (initialEnvironment o)
 
-instance SVGFloat n => Monoid (Render SVG V2 n) where
+instance Monoid (Render SVG V2 n) where
   mempty = R $ return mempty
   R r1 `mappend` R r2_ = R $ do
     svg1 <- r1
@@ -199,7 +205,7 @@ instance SVGFloat n => Monoid (Render SVG V2 n) where
 --
 renderSvgWithClipping :: forall n. SVGFloat n
                       => T.Text
-                      -> SvgM          -- ^ Input SVG
+                      -> Element       -- ^ Input SVG
                       -> Style V2 n    -- ^ Styles
                       -> SvgRenderM n  -- ^ Resulting svg
 
@@ -208,7 +214,7 @@ renderSvgWithClipping prefix svg s =
     Nothing    -> return svg
     Just paths -> renderClips paths
   where
-    renderClips :: SVGFloat n => [Path V2 n] -> SvgRenderM n
+    renderClips :: [Path V2 n] -> SvgRenderM n
     renderClips []     = return svg
     renderClips (p:ps) = do
       clipPathId += 1
@@ -231,53 +237,72 @@ lineTextureDefs s = do
 
 instance SVGFloat n => Backend SVG V2 n where
   newtype Render  SVG V2 n = R (SvgRenderM n)
-  type    Result  SVG V2 n = SvgM
+  type    Result  SVG V2 n = Element
   data    Options SVG V2 n = SVGOptions
-    { _size           :: SizeSpec V2 n   -- ^ The requested size.
-    , _svgDefinitions :: Maybe SvgM
+    { _size            :: SizeSpec V2 n   -- ^ The requested size.
+    , _svgDefinitions  :: Maybe Element
                           -- ^ Custom definitions that will be added to the @defs@
                           --   section of the output.
-    , _idPrefix       :: T.Text
+    , _idPrefix        :: T.Text
+    , _svgAttributes   :: [Attribute]
+                          -- ^ Attriubtes to apply to the entire svg element.
+    , _generateDoctype :: Bool
     }
 
+  renderRTree :: SVG -> Options SVG V2 n -> RTree SVG V2 n Annotation -> Result SVG V2 n
   renderRTree _ opts rt = runRenderM (opts ^.idPrefix) svgOutput
     where
       svgOutput = do
         let R r    = rtree (splitTextureFills rt)
             V2 w h = specToSize 100 (opts^.sizeSpec)
         svg <- r
-        return $ R.svgHeader w h (opts^.svgDefinitions) svg
+        return $ R.svgHeader w h (opts^.svgDefinitions)
+                                 (opts^.svgAttributes)
+                                 (opts^.generateDoctype) svg
 
-  adjustDia c opts d = adjustDia2D sizeSpec c opts (d # reflectY)
+  adjustDia c opts d = ( sz, t <> reflectionY, d' ) where
+    (sz, t, d') = adjustDia2D sizeSpec c opts (d # reflectY)
 
 rtree :: SVGFloat n => RTree SVG V2 n Annotation -> Render SVG V2 n
 rtree (Node n rs) = case n of
   RPrim p                 -> render SVG p
   RStyle sty              -> R $ local (over style (<> sty)) r
-  RAnnot (OpacityGroup o) -> R $ g_ [opacity_ $ toText o] <$> r
-  RAnnot (Href uri)       -> R $ a_ [xlinkHref_ $ T.pack uri] <$> r
+  RAnnot (OpacityGroup o) -> R $ g_ [Opacity_ <<- toText o] <$> r
+  RAnnot (Href uri)       -> R $ a_ [XlinkHref_ <<- T.pack uri] <$> r
   _                       -> R r
   where
     R r = foldMap rtree rs
 
 -- | Lens onto the size of the svg options.
-sizeSpec :: SVGFloat n => Lens' (Options SVG V2 n) (SizeSpec V2 n)
+sizeSpec :: Lens' (Options SVG V2 n) (SizeSpec V2 n)
 sizeSpec f opts = f (_size opts) <&> \s -> opts { _size = s }
 
 -- | Lens onto the svg definitions of the svg options.
-svgDefinitions :: SVGFloat n => Lens' (Options SVG V2 n) (Maybe SvgM)
+svgDefinitions :: Lens' (Options SVG V2 n) (Maybe Element)
 svgDefinitions f opts =
   f (_svgDefinitions opts) <&> \ds -> opts { _svgDefinitions = ds }
 
 -- | Lens onto the idPrefix of the svg options. This is the prefix given
 --   to clipping paths to distinguish them from other svg files in the
 --   same web page.
-idPrefix :: SVGFloat n => Lens' (Options SVG V2 n) T.Text
+idPrefix :: Lens' (Options SVG V2 n) T.Text
 idPrefix f opts = f (_idPrefix opts) <&> \i -> opts { _idPrefix = i }
 
+-- | Lens onto the svgAttributes field of the svg options. This field
+--   is provided to supply SVG attributes to the entire diagram.
+svgAttributes :: Lens' (Options SVG V2 n) [Attribute]
+svgAttributes f opts =
+  f (_svgAttributes opts) <&> \ds -> opts { _svgAttributes = ds }
+
+-- | Lens onto the generateDoctype field of the svg options. Set
+--   to False if you don't want a doctype tag included in the output.
+generateDoctype :: Lens' (Options SVG V2 n) Bool
+generateDoctype f opts =
+  f (_generateDoctype opts) <&> \ds -> opts { _generateDoctype = ds }
+
 -- paths ---------------------------------------------------------------
 
-attributedRender :: SVGFloat n => SvgM -> SvgRenderM n
+attributedRender :: SVGFloat n => Element -> SvgRenderM n
 attributedRender svg = do
   SvgRenderState _idClip idFill idLine <- get
   Environment sty preT <- ask
@@ -285,8 +310,8 @@ attributedRender svg = do
   lineGradDefs <- lineTextureDefs sty
   fillGradDefs <- fillTextureDefs sty
   return $ do
-    defs_ $ fillGradDefs >> lineGradDefs
-    g_ (R.renderStyles idFill idLine sty) clippedSvg
+    let gDefs = mappend fillGradDefs lineGradDefs
+    gDefs `mappend` g_ (R.renderStyles idFill idLine sty) clippedSvg
 
 instance SVGFloat n => Renderable (Path V2 n) SVG where
   render _ = R . attributedRender . R.renderPath
@@ -300,11 +325,11 @@ instance SVGFloat n => Renderable (DImage n Embedded) SVG where
 -- | Render a diagram as an SVG, writing to the specified output file
 --   and using the requested size.
 renderSVG :: SVGFloat n => FilePath -> SizeSpec V2 n -> QDiagram SVG V2 n Any -> IO ()
-renderSVG outFile spec = renderSVG' outFile (SVGOptions spec Nothing (mkPrefix outFile))
+renderSVG outFile spec = renderSVG' outFile (SVGOptions spec Nothing (mkPrefix outFile) [] True)
 
 -- | Render a diagram as a pretty printed SVG.
 renderPretty :: SVGFloat n => FilePath -> SizeSpec V2 n -> QDiagram SVG V2 n Any -> IO ()
-renderPretty outFile spec = renderPretty' outFile (SVGOptions spec Nothing (mkPrefix outFile))
+renderPretty outFile spec = renderPretty' outFile (SVGOptions spec Nothing (mkPrefix outFile)[] True)
 
 -- Create a prefile using the basename of the output file. Only standard
 -- letters are considered.
@@ -334,11 +359,11 @@ loadImageSVG fp = do
     let pic t d = return $ image (DImage (ImageNative (Img t d))
                                    (dynamicMap imageWidth dyn)
                                    (dynamicMap imageHeight dyn) mempty)
-    if pngHeader `SBS.isPrefixOf` raw then pic 'P' dat else do
-    if jpgHeader `SBS.isPrefixOf` raw then pic 'J' dat else do
-    case dyn of
-      (ImageYCbCr8 _) -> pic 'J' dat
-      _               -> pic 'P' =<< eIO (encodeDynamicPng dyn)
+    if | pngHeader `SBS.isPrefixOf` raw -> pic 'P' dat
+       | jpgHeader `SBS.isPrefixOf` raw -> pic 'J' dat
+       | otherwise -> case dyn of
+           (ImageYCbCr8 _) -> pic 'J' dat
+           _               -> pic 'P' =<< eIO (encodeDynamicPng dyn)
   where pngHeader :: SBS.ByteString
         pngHeader = SBS.pack [137, 80, 78, 71, 13, 10, 26, 10]
         jpgHeader :: SBS.ByteString
@@ -354,6 +379,12 @@ instance SVGFloat n => Renderable (DImage n (Native Img)) SVG where
           _   -> fail   "Unknown mime type while rendering image"
     return $ R.renderDImage di $ R.dataUri mime d
 
-instance (Hashable n, SVGFloat n) => Hashable (Options SVG V2 n) where
-  hashWithSalt s  (SVGOptions sz defs _) = s `hashWithSalt` sz `hashWithSalt` ds
-    where ds = fmap renderBS defs
+instance Hashable n => Hashable (Options SVG V2 n) where
+  hashWithSalt s  (SVGOptions sz defs ia sa gd) =
+    s  `hashWithSalt`
+    sz `hashWithSalt`
+    ds `hashWithSalt`
+    ia `hashWithSalt`
+    sa `hashWithSalt`
+    gd
+      where ds = fmap renderBS defs
