@@ -20,7 +20,7 @@
 
 module Graphics.Rendering.SVG
     ( SVGFloat
-    , SvgM
+    , Element
     , AttributeValue
     , svgHeader
     , renderPath
@@ -51,8 +51,7 @@ import           Data.Monoid
 import           Diagrams.Core.Transform     (matrixHomRep)
 
 -- from diagrams-lib
-import           Diagrams.Prelude            hiding (Attribute, Render, with,
-                                              (<>))
+import           Diagrams.Prelude            hiding (Attribute, Render, with, (<>))
 import           Diagrams.TwoD.Path          (getFillRule)
 import           Diagrams.TwoD.Text
 
@@ -61,7 +60,7 @@ import           Data.Text                   (pack)
 import qualified Data.Text                   as T
 
 -- from lucid-svg
-import           Lucid.Svg                   hiding (renderText)
+import           Graphics.Svg                hiding (renderText)
 
 -- from base64-bytestring, bytestring
 import qualified Data.ByteString.Base64.Lazy as BS64
@@ -70,9 +69,6 @@ import qualified Data.ByteString.Lazy.Char8  as BS8
 -- from JuicyPixels
 import           Codec.Picture
 
--- from same package
-import Diagrams.Backend.SVG.Attributes       (SvgId(..),SvgClass(..))
-
 -- | Constaint on number type that diagrams-svg can use to render an SVG. This
 --   includes the common number types: Double, Float
 type SVGFloat n = (Show n, TypeableFloat n)
@@ -80,9 +76,6 @@ type SVGFloat n = (Show n, TypeableFloat n)
 --   showFFloat :: RealFloat a => Maybe Int -> a -> ShowS
 -- or something similar for all numbers so we need TypeableFloat constraint.
 
--- | The Svg monad, synonym for @HtmlT m ()@ Lucid.Base.
-type SvgM = Svg ()
-
 type AttributeValue = T.Text
 
 getNumAttr :: AttributeClass (a n) => (a n -> t) -> Style v n -> Maybe t
@@ -90,24 +83,29 @@ getNumAttr f = (f <$>) . getAttr
 
 -- | @svgHeader w h defs s@: @w@ width, @h@ height,
 --   @defs@ global definitions for defs sections, @s@ actual SVG content.
-svgHeader :: SVGFloat n => n -> n -> Maybe SvgM -> SvgM -> SvgM
-svgHeader w h defines s =  doctype_ <> with (svg11_ (defs_  ds <> s))
-  [ width_  (toText w)
-  , height_ (toText h)
-  , font_size_ "1"
-  , viewBox_ (pack . unwords $ map show ([0, 0, round w, round h] :: [Int]))
-  , stroke_ "rgb(0,0,0)"
-  , stroke_opacity_ "1" ]
+svgHeader :: SVGFloat n => n -> n -> Maybe Element -> [Attribute] -> Bool
+                        -> Element -> Element
+svgHeader w h defines attributes genDoctype s =
+  dt <> with (svg11_ (defs_ [] ds <> s))
+    ([ Width_ <<- toText w
+     , Height_ <<- toText h
+     , Font_size_ <<- "1"
+     , ViewBox_ <<- (pack . unwords $ map show ([0, 0, round w, round h] :: [Int]))
+     , Stroke_ <<- "rgb(0,0,0)"
+     , Stroke_opacity_ <<- "1" ]
+     ++ attributes )
   where
     ds = fromMaybe mempty defines
+    dt = if genDoctype then doctype else mempty
 
-renderPath :: SVGFloat n => Path V2 n -> SvgM
-renderPath trs = if makePath == T.empty then mempty else path_ [d_ makePath]
+renderPath :: SVGFloat n => Path V2 n -> Element
+renderPath trs = if makePath == T.empty then mempty else path_ [D_ <<- makePath]
   where
     makePath = foldMap renderTrail (op Path trs)
 
 renderTrail :: SVGFloat n => Located (Trail V2 n) -> AttributeValue
-renderTrail (viewLoc -> (P (V2 x y), t)) = mA x y <> withTrail renderLine renderLoop t
+renderTrail (viewLoc -> (P (V2 x y), t)) =
+  mA x y <> withTrail renderLine renderLoop t
   where
     renderLine = foldMap renderSeg . lineSegments
     renderLoop lp =
@@ -127,34 +125,34 @@ renderSeg (Cubic  (V2 x0 y0)
                   (V2 x1 y1)
                   (OffsetClosed (V2 x2 y2))) = cR x0 y0 x1 y1 x2 y2
 
-renderClip :: SVGFloat n => Path V2 n -> T.Text -> Int -> SvgM -> SvgM
+renderClip :: SVGFloat n => Path V2 n -> T.Text -> Int -> Element -> Element
 renderClip p prefix ident svg = do
-  defs_ $ clipPath_ [id_ (clipPathId ident)] (renderPath p)
-  g_  [clip_path_ $ ("url(#" <> clipPathId ident <> ")")] svg
+     defs_ [] $ clipPath_ [Id_ <<- (clipPathId ident)] (renderPath p)
+  <> g_  [Clip_path_ <<- ("url(#" <> clipPathId ident <> ")")] svg
     where
       clipPathId i = prefix <> "myClip" <> (pack . show $ i)
 
-renderStop :: SVGFloat n => GradientStop n -> SvgM
+renderStop :: SVGFloat n => GradientStop n -> Element
 renderStop (GradientStop c v)
-  = stop_ [ stop_color_   (colorToRgbText c)
-          , offset_      (toText v)
-          , stop_opacity_ (toText $ colorToOpacity c) ]
+  = stop_ [ Stop_color_ <<- (colorToRgbText c)
+          , Offset_ <<- (toText v)
+          , Stop_opacity_ <<- (toText $ colorToOpacity c) ]
 
 spreadMethodText :: SpreadMethod -> AttributeValue
 spreadMethodText GradPad      = "pad"
 spreadMethodText GradReflect  = "reflect"
 spreadMethodText GradRepeat   = "repeat"
 
-renderLinearGradient :: SVGFloat n => LGradient n -> Int -> SvgM
+renderLinearGradient :: SVGFloat n => LGradient n -> Int -> Element
 renderLinearGradient g i = linearGradient_
-    [ id_ (pack $ "gradient" ++ show i)
-    , x1_ (toText x1)
-    , y1_ (toText y1)
-    , x2_ (toText x2)
-    , y2_ (toText y2)
-    , gradientTransform_ mx
-    , gradientUnits_ "userSpaceOnUse"
-    , spreadMethod_ (spreadMethodText (g ^. lGradSpreadMethod)) ]
+    [ Id_ <<- (pack $ "gradient" ++ show i)
+    , X1_ <<- toText x1
+    , Y1_ <<- toText y1
+    , X2_ <<- toText x2
+    , Y2_ <<- toText y2
+    , GradientTransform_ <<- mx
+    , GradientUnits_ <<- "userSpaceOnUse"
+    , SpreadMethod_ <<- spreadMethodText (g ^. lGradSpreadMethod) ]
     $ foldMap renderStop (g^.lGradStops)
   where
     mx = matrix a1 a2 b1 b2 c1 c2
@@ -162,17 +160,17 @@ renderLinearGradient g i = linearGradient_
     P (V2 x1 y1) = g ^. lGradStart
     P (V2 x2 y2) = g ^. lGradEnd
 
-renderRadialGradient :: SVGFloat n => RGradient n -> Int -> SvgM
+renderRadialGradient :: SVGFloat n => RGradient n -> Int -> Element
 renderRadialGradient g i = radialGradient_
-    [ id_ (pack $ "gradient" ++ show i)
-    , r_ (toText (g ^. rGradRadius1))
-    , cx_ (toText cx)
-    , cy_ (toText cy)
-    , fx_ (toText fx)
-    , fy_ (toText fy)
-    , gradientTransform_ mx
-    , gradientUnits_ "userSpaceOnUse"
-    , spreadMethod_ (spreadMethodText (g ^. rGradSpreadMethod)) ]
+    [ Id_ <<- (pack $ "gradient" ++ show i)
+    , R_  <<- toText (g ^. rGradRadius1)
+    , Cx_ <<- toText cx
+    , Cy_ <<- toText cy
+    , Fx_ <<- toText fx
+    , Fy_ <<- toText fy
+    , GradientTransform_ <<- mx
+    , GradientUnits_ <<- "userSpaceOnUse"
+    , SpreadMethod_ <<- spreadMethodText (g ^. rGradSpreadMethod) ]
     ( foldMap renderStop ss )
   where
     mx = matrix a1 a2 b1 b2 c1 c2
@@ -192,51 +190,51 @@ renderRadialGradient g i = radialGradient_
     ss = zipWith (\gs sf -> gs & stopFraction .~ sf ) gradStops stopFracs
 
 -- Create a gradient element so that it can be used as an attribute value for fill.
-renderFillTextureDefs :: SVGFloat n => Int -> Style v n -> SvgM
+renderFillTextureDefs :: SVGFloat n => Int -> Style v n -> Element
 renderFillTextureDefs i s =
   case getNumAttr getFillTexture s of
-    Just (LG g) -> renderLinearGradient g i
-    Just (RG g) -> renderRadialGradient g i
+    Just (LG g) -> defs_ [] $ renderLinearGradient g i
+    Just (RG g) -> defs_ [] $ renderRadialGradient g i
     _           -> mempty
 
 -- Render the gradient using the id set up in renderFillTextureDefs.
 renderFillTexture :: SVGFloat n => Int -> Style v n -> [Attribute]
 renderFillTexture ident s = case getNumAttr getFillTexture s of
-  Just (SC (SomeColor c)) -> renderTextAttr fill_ fillColorRgb <>
-                             renderAttr fill_opacity_ fillColorOpacity
+  Just (SC (SomeColor c)) -> renderTextAttr Fill_ fillColorRgb <>
+                             renderAttr Fill_opacity_ fillColorOpacity
     where
       fillColorRgb     = Just $ colorToRgbText c
       fillColorOpacity = Just $ colorToOpacity c
-  Just (LG _) -> [fill_ ("url(#gradient" <> (pack . show $ ident)
-                                         <> ")"), fill_opacity_ "1"]
-  Just (RG _) -> [fill_ ("url(#gradient" <> (pack . show $ ident)
-                                         <> ")"), fill_opacity_ "1"]
+  Just (LG _) -> [Fill_ <<- ("url(#gradient" <> (pack . show $ ident)
+                                             <> ")"), Fill_opacity_ <<- "1"]
+  Just (RG _) -> [Fill_ <<- ("url(#gradient" <> (pack . show $ ident)
+                                             <> ")"), Fill_opacity_ <<- "1"]
   Nothing     -> []
 
-renderLineTextureDefs :: SVGFloat n => Int -> Style v n -> SvgM
+renderLineTextureDefs :: SVGFloat n => Int -> Style v n -> Element
 renderLineTextureDefs i s =
   case getNumAttr getLineTexture s of
-    Just (LG g) -> renderLinearGradient g i
-    Just (RG g) -> renderRadialGradient g i
+    Just (LG g) -> defs_ [] $ renderLinearGradient g i
+    Just (RG g) -> defs_ [] $ renderRadialGradient g i
     _           -> mempty
 
 renderLineTexture :: SVGFloat n => Int -> Style v n -> [Attribute]
 renderLineTexture ident s = case getNumAttr getLineTexture s of
-  Just (SC (SomeColor c)) -> renderTextAttr stroke_ lineColorRgb <>
-                             renderAttr stroke_opacity_ lineColorOpacity
+  Just (SC (SomeColor c)) -> renderTextAttr Stroke_ lineColorRgb <>
+                             renderAttr Stroke_opacity_ lineColorOpacity
     where
       lineColorRgb     = Just $ colorToRgbText c
       lineColorOpacity = Just $ colorToOpacity c
-  Just (LG _) -> [stroke_ ("url(#gradient" <> (pack . show $ ident)
-                                           <> ")"), stroke_opacity_ "1"]
-  Just (RG _) -> [stroke_ ("url(#gradient" <> (pack . show $ ident)
-                                           <> ")"), stroke_opacity_ "1"]
+  Just (LG _) -> [Stroke_ <<- ("url(#gradient" <> (pack . show $ ident)
+                                               <> ")"), Stroke_opacity_ <<- "1"]
+  Just (RG _) -> [Stroke_ <<- ("url(#gradient" <> (pack . show $ ident)
+                                               <> ")"), Stroke_opacity_ <<- "1"]
   Nothing     -> []
 
 dataUri :: String -> BS8.ByteString -> AttributeValue
 dataUri mime dat = pack $ "data:"++mime++";base64," ++ BS8.unpack (BS64.encode dat)
 
-renderDImageEmb :: SVGFloat n => DImage n Embedded -> SvgM
+renderDImageEmb :: SVGFloat n => DImage n Embedded -> Element
 renderDImageEmb di@(DImage (ImageRaster dImg) _ _ _) =
   renderDImage di $ dataUri "image/png" img
   where
@@ -244,13 +242,13 @@ renderDImageEmb di@(DImage (ImageRaster dImg) _ _ _) =
             Left str   -> error str
             Right img' -> img'
 
-renderDImage :: SVGFloat n => DImage n any -> AttributeValue -> SvgM
+renderDImage :: SVGFloat n => DImage n any -> AttributeValue -> Element
 renderDImage (DImage _ w h tr) uridata =
   image_
-    [ transform_ transformMatrix
-    , width_  (pack . show $ w)
-    , height_ (pack . show $ h)
-    , xlinkHref_ uridata ]
+    [ Transform_ <<- transformMatrix
+    , Width_ <<-  (pack . show $ w)
+    , Height_ <<- (pack . show $ h)
+    , XlinkHref_ <<- uridata ]
   where
     [[a,b],[c,d],[e,f]] = matrixHomRep (tr `mappend` reflectionY
                                            `mappend` tX `mappend` tY)
@@ -258,14 +256,14 @@ renderDImage (DImage _ w h tr) uridata =
     tX = translationX $ fromIntegral (-w)/2
     tY = translationY $ fromIntegral (-h)/2
 
-renderText :: SVGFloat n => Text n -> SvgM
+renderText :: SVGFloat n => Text n -> Element
 renderText (Text tt tAlign str) =
   text_
-    [ transform_ transformMatrix
-    , dominant_baseline_ vAlign
-    , text_anchor_ hAlign
-    , stroke_ "none" ]
-    $ toHtml str
+    [ Transform_ <<- transformMatrix
+    , Dominant_baseline_ <<- vAlign
+    , Text_anchor_ <<- hAlign
+    , Stroke_ <<- "none" ]
+    $ toElement str
  where
   vAlign = case tAlign of
              BaselineText -> "alphabetic"
@@ -297,48 +295,37 @@ renderStyles fillId lineId s = concatMap ($ s) $
   , renderFontSlant
   , renderFontWeight
   , renderFontFamily
-  , renderSvgId
-  , renderSvgClass
   , renderMiterLimit ]
 
-
-renderSvgId :: SVGFloat n => Style v n -> [Attribute]
-renderSvgId s = renderTextAttr id_ svgIdAttr
- where svgIdAttr = pack . getSvgId <$> getAttr s
-
-renderSvgClass :: SVGFloat n => Style v n -> [Attribute]
-renderSvgClass s = renderTextAttr class_ svgClassAttr
- where svgClassAttr = pack . getSvgClass <$> getAttr s
-
-renderMiterLimit :: SVGFloat n => Style v n -> [Attribute]
-renderMiterLimit s = renderAttr stroke_miterlimit_ miterLimit
+renderMiterLimit :: Style v n -> [Attribute]
+renderMiterLimit s = renderAttr Stroke_miterlimit_ miterLimit
  where miterLimit = getLineMiterLimit <$> getAttr s
 
-renderOpacity :: SVGFloat n => Style v n -> [Attribute]
-renderOpacity s = renderAttr opacity_ o
+renderOpacity :: Style v n -> [Attribute]
+renderOpacity s = renderAttr Opacity_ o
  where o = getOpacity <$> getAttr s
 
-renderFillRule :: SVGFloat n => Style v n -> [Attribute]
-renderFillRule s = renderTextAttr fill_rule_ fr
+renderFillRule :: Style v n -> [Attribute]
+renderFillRule s = renderTextAttr Fill_rule_ fr
   where fr = (fillRuleToText . getFillRule) <$> getAttr s
         fillRuleToText :: FillRule -> AttributeValue
         fillRuleToText Winding = "nonzero"
         fillRuleToText EvenOdd = "evenodd"
 
 renderLineWidth :: SVGFloat n => Style v n -> [Attribute]
-renderLineWidth s = renderAttr stroke_width_ lWidth
+renderLineWidth s = renderAttr Stroke_width_ lWidth
   where lWidth = getNumAttr getLineWidth s
 
-renderLineCap :: SVGFloat n => Style v n -> [Attribute]
-renderLineCap s = renderTextAttr stroke_linecap_ lCap
+renderLineCap :: Style v n -> [Attribute]
+renderLineCap s = renderTextAttr Stroke_linecap_ lCap
   where lCap = (lineCapToText . getLineCap) <$> getAttr s
         lineCapToText :: LineCap -> AttributeValue
         lineCapToText LineCapButt   = "butt"
         lineCapToText LineCapRound  = "round"
         lineCapToText LineCapSquare = "square"
 
-renderLineJoin :: SVGFloat n => Style v n -> [Attribute]
-renderLineJoin s = renderTextAttr stroke_linejoin_ lj
+renderLineJoin :: Style v n -> [Attribute]
+renderLineJoin s = renderTextAttr Stroke_linejoin_ lj
   where lj = (lineJoinToText . getLineJoin) <$> getAttr s
         lineJoinToText :: LineJoin -> AttributeValue
         lineJoinToText LineJoinMiter = "miter"
@@ -346,26 +333,26 @@ renderLineJoin s = renderTextAttr stroke_linejoin_ lj
         lineJoinToText LineJoinBevel = "bevel"
 
 renderDashing :: SVGFloat n => Style v n -> [Attribute]
-renderDashing s = renderTextAttr stroke_dasharray_ arr <>
-                  renderAttr stroke_dashoffset_ dOffset
+renderDashing s = renderTextAttr Stroke_dasharray_ arr <>
+                  renderAttr Stroke_dashoffset_ dOffset
  where
   getDasharray  (Dashing a _) = a
   getDashoffset (Dashing _ o) = o
-  dashArrayToStr              = intercalate "," . map show
+  dashArrayToStr = intercalate "," . map show
   -- Ignore dashing if dashing array is empty
   checkEmpty (Just (Dashing [] _)) = Nothing
-  checkEmpty other                 = other
-  dashing'                    = checkEmpty $ getNumAttr getDashing s
-  arr                         = (pack . dashArrayToStr . getDasharray) <$> dashing'
-  dOffset                     = getDashoffset <$> dashing'
+  checkEmpty other = other
+  dashing' = checkEmpty $ getNumAttr getDashing s
+  arr = (pack . dashArrayToStr . getDasharray) <$> dashing'
+  dOffset = getDashoffset <$> dashing'
 
 renderFontSize :: SVGFloat n => Style v n -> [Attribute]
-renderFontSize s = renderTextAttr font_size_ fs
+renderFontSize s = renderTextAttr Font_size_ fs
  where
   fs = pack <$> getNumAttr ((++ "px") . show . getFontSize) s
 
-renderFontSlant :: SVGFloat n => Style v n -> [Attribute]
-renderFontSlant s = renderTextAttr font_style_ fs
+renderFontSlant :: Style v n -> [Attribute]
+renderFontSlant s = renderTextAttr Font_style_ fs
  where
   fs = (fontSlantAttr . getFontSlant) <$> getAttr s
   fontSlantAttr :: FontSlant -> AttributeValue
@@ -373,25 +360,27 @@ renderFontSlant s = renderTextAttr font_style_ fs
   fontSlantAttr FontSlantOblique = "oblique"
   fontSlantAttr FontSlantNormal  = "normal"
 
-renderFontWeight :: SVGFloat n => Style v n -> [Attribute]
-renderFontWeight s = renderTextAttr font_weight_ fw
+renderFontWeight :: Style v n -> [Attribute]
+renderFontWeight s = renderTextAttr Font_weight_ fw
  where
   fw = (fontWeightAttr . getFontWeight) <$> getAttr s
   fontWeightAttr :: FontWeight -> AttributeValue
   fontWeightAttr FontWeightNormal = "normal"
   fontWeightAttr FontWeightBold   = "bold"
 
-renderFontFamily :: SVGFloat n => Style v n -> [Attribute]
-renderFontFamily s = renderTextAttr  font_family_ ff
+
+renderFontFamily :: Style v n -> [Attribute]
+renderFontFamily s = renderTextAttr Font_family_ ff
  where
   ff = (pack . getFont) <$> getAttr s
 
 -- | Render a style attribute if available, empty otherwise.
-renderAttr :: Show s => (AttributeValue -> Attribute) -> Maybe s -> [Attribute]
-renderAttr attr valM = maybe [] (\v -> [attr (pack . show $ v)]) valM
+renderAttr :: Show s => AttrTag -> Maybe s -> [Attribute]
+renderAttr attr valM = maybe [] (\v -> [(bindAttr attr) (pack . show $ v)]) valM
 
-renderTextAttr :: (AttributeValue -> Attribute) -> Maybe AttributeValue -> [Attribute]
-renderTextAttr attr valM = maybe [] (\v -> [attr v]) valM
+-- renderTextAttr :: (AttributeValue -> Attribute) -> Maybe AttributeValue -> [Attribute]
+renderTextAttr :: AttrTag -> Maybe AttributeValue -> [Attribute]
+renderTextAttr attr valM = maybe [] (\v -> [(bindAttr attr) v]) valM
 
 colorToRgbText :: forall c . Color c => c -> AttributeValue
 colorToRgbText c = T.concat
